open Sys;;
open Unix;;

module G = 
  Gethostbyname.Make (struct
    let addr_number = 20 (*1000*) 
    let not_found = 10 
  end);;

module DNS = 
  Dns.Make (struct
    let resolve = G.gethostbyname
    let max_active_threads = 13
  end ) ;;

let resolve name = 
  try 
    let entry = DNS.resolve name in
    if entry.h_name <> name then
      failwith "Resolution failure"
  with Not_found -> ();;

let test num = 
  for i = 0 to 15 (* 1500 *)  do
    let name = 
      if num mod 100 == 0 then
        string_of_int (Random.int G.addr_number)
      else
        string_of_int (i mod G.addr_number)
    in
    resolve name
  done;;

let test1 () =
  let threads = Array.init 3 (*100*) (Thread.create test) in
  Array.iter Thread.join threads;
  (* pour etre sur d'avoir appele tout le monde *)
  for i = 0 to G.addr_number -1 do resolve (string_of_int i) done;
  G.check();;

exception Timeout;;
let timeout _ = 
  Printf.printf
    "Timeout during test (may be deadlocked)\n%!";
  exit 1;;
let _ = signal sigalrm (Signal_handle timeout);;
let _ = ignore (sigprocmask SIG_BLOCK [ sigalrm ]);;
let settimeout t = 
  if t > 0. then 
     let _ = sigprocmask SIG_UNBLOCK [ sigalrm ] in 
     let _ = alarm (max 1 (int_of_float t)) in ()
  else 
     let _ = sigprocmask SIG_UNBLOCK [ sigalrm ] in 
     let _ = alarm 0 in ();;

let exec i text max_time f =
  let start = gettimeofday () in
  (* should be adjusted, since some requests are treated synchronously *)
  let expected_time = max_time /. float i in
  Printf.printf
    "Starting test %s, maximum time: %.2f, expected time: %.2f\n%!"
    text max_time expected_time;
  settimeout (max_time *. 1.3);
  f ();
  settimeout 0.;
  let time = gettimeofday() -. start in
  if max_time > 0. && time > 2. *. expected_time then
    Printf.printf
      "Test %s  ends after %.2f ---that's %.2f times too long!\n%!" 
      text time (time /. expected_time)
  else
    Printf.printf "Test %s ends after %.2f ---OK\n%!" 
      text time;;

module G2 = 
  Gethostbyname.Make (struct
    let addr_number = 100 (* 1000 *)
    let not_found = 10 
  end)

let run_test i = 
  let module DNS = 
    Dns.Make (struct
      let resolve = G2.gethostbyname
      let max_active_threads = i
    end ) in
  let test () =
    for i = 0 to G2.addr_number - 1  do
       try ignore (DNS.try_resolve (string_of_int i)) with _ -> ()
    done;
    for i = 0 to G2.addr_number - 1 do
      try ignore (DNS.resolve (string_of_int i)) with _ -> ()
    done;
    G2.check ();
    G2.reset () in
  let text = Printf.sprintf "2 with %d threads" i in
  exec i text !G2.total test;;

let _ = 
  try exec 1 "1" 0. test1 with _ -> print_endline "Error in first test";;

for i = 1 to 10 do
  run_test i
done;;

module G3 = 
  Gethostbyname.Make (struct
    let addr_number = 1000
    let not_found = 10 
  end);;

let test3 () =
  let module DNS = 
    Dns.Make (struct
      let resolve = G3.gethostbyname
      let max_active_threads = 10
    end ) in
  let k = G3.addr_number / 2 in
  let l = k / 10 in
  let perform n =
    for i = 0 to l - 1 do
      try ignore (DNS.resolve (string_of_int (n*l+i))) with _ -> ()
    done in
  let threads = Array.init 10 (Thread.create perform) in
  for i = k to G3.addr_number - 1 do
    try ignore (DNS.try_resolve (string_of_int i)) with _ -> ()
  done;
  for i = k to G3.addr_number - 1 do
    try ignore (DNS.resolve (string_of_int i)) with _ -> ()
  done;
  Array.iter Thread.join threads;
  G3.check ();
  G3.reset ();;

let _ =
  try exec 10 "3" (!G3.total /. 2.)  test3 with _ -> print_endline "Error in first test";;


